__version__ = "0.1.23"
__description__ = "Brain++ tools devloped by Yang Lei."
__license__ = "MIT"
__author__ = "yanglei"
__author_email__ = "ylxx@live.com"
__maintainer__ = "yanglei"
__maintainer_email__ = "ylxx@live.com"
__github_username__ = "yanglei"
__url__ = "https://git-core.megvii-inc.com/yanglei/brainpp_yl"
__support__ = "https://git-core.megvii-inc.com/yanglei/brainpp_yl/issues"
__classifiers__ = [
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
]
